@@ -136,16 +136,20 @@ permute (struct permute_info *pinfo, unsigned dimnum)
 
       {
 	void *pointaddr;
-
+        
 	switch (pinfo->type)
 	  {
-	  case STRSXP: case VECSXP:
-	    pointaddr = &((SEXPREC **)pinfo->buf)[offset];
-	    break;
+	  case STRSXP: 
+          case VECSXP:
+            pointaddr = &((SEXPREC **)pinfo->buf)[offset];
+            break;
 	  case REALSXP:
 	    pointaddr = &((double *)pinfo->buf)[offset];
 	    break;
-	  case INTSXP: case LGLSXP:
+	  case INTSXP: 
+	    pointaddr = &((int *)pinfo->buf)[offset];
+            break;
+          case LGLSXP:
 	    pointaddr = &((int *)pinfo->buf)[offset];
 	    break;
 	  default:
@@ -213,6 +217,20 @@ get_string_type (SEXP call, SEXP vec)
   return stid;
 }
 
+static hid_t
+make_boolean_type (SEXP call)
+{
+  hid_t tid;
+
+  if ((tid = H5Tcopy (H5T_NATIVE_UINT)) < 0)
+    errorcall (call, "Cannot copy unsigned integer type");
+  if (H5Tset_precision (tid, 1) < 0)
+    errorcall (call, "Cannot set precision of boolean type");
+  if (H5Tset_size (tid, 1) < 0)
+    errorcall (call, "Cannot set size of boolean type");
+  return tid;
+}
+
 static void
 load_scalar (SEXP call, hid_t dataset, hid_t space, SEXP obj)
 {
@@ -229,21 +247,21 @@ load_scalar (SEXP call, hid_t dataset, hid_t space, SEXP obj)
   switch (type) {
 
   case STRSXP:
-      memtid = make_sexp_ref_type (call);
-      buf = STRING_PTR (obj);
-      break;
+    memtid = make_sexp_ref_type (call);
+    buf = STRING_PTR (obj);
+    break;
   case REALSXP:
-      memtid = H5T_NATIVE_DOUBLE;
-      buf = REAL (obj);
-      break;
+    memtid = H5T_NATIVE_DOUBLE;
+    buf = REAL (obj);
+    break;
   case INTSXP:
-      memtid = H5T_NATIVE_INT;
-      buf = INTEGER (obj);
-      break;
+    memtid = H5T_NATIVE_INT;
+    buf = INTEGER (obj);
+    break;
   case LGLSXP:
-      memtid = H5T_NATIVE_UINT;
-      buf = INTEGER (obj);
-      break;
+    memtid = make_boolean_type (call);
+    buf = LOGICAL (obj);
+    break;
   default:
       errorcall (call, "Can't get type for R type: %d (IO)", type);
       /* unreached (-Wall): */ memtid = tid;  buf = &tid;
@@ -256,6 +274,10 @@ load_scalar (SEXP call, hid_t dataset, hid_t space, SEXP obj)
                H5P_DEFAULT,
                buf) < 0)
     errorcall (call, "Unable to read dataset");
+
+  if (type == LGLSXP)
+    if (H5Tclose (memtid) < 0)
+      errorcall (call, "can't close boolean type");
 }
 
 static void
@@ -272,28 +294,27 @@ vector_io (SEXP call, int writeflag, hid_t dataset, hid_t space, SEXP obj)
     errorcall (call, "Unable to get type for dataset");
 
   switch (type) {
-
   case STRSXP:
-      memtid = make_sexp_ref_type (call);
-      buf = STRING_PTR (obj);
-      break;
+    memtid = make_sexp_ref_type (call);
+    buf = STRING_PTR (obj);
+    break;
   case REALSXP:
-      memtid = H5T_NATIVE_DOUBLE;
-      buf = REAL (obj);
-      break;
+    memtid = H5T_NATIVE_DOUBLE;
+    buf = REAL (obj);
+    break;
   case INTSXP:
-      memtid = H5T_NATIVE_INT;
-      buf = INTEGER (obj);
-      break;
+    memtid = H5T_NATIVE_INT;
+    buf = INTEGER (obj);
+    break;
   case LGLSXP:
-      memtid = H5T_NATIVE_UINT;
-      buf = INTEGER (obj);
-      break;
+    memtid = make_boolean_type (call);
+    buf = LOGICAL (obj);
+    break;
   default:
-      errorcall (call, "Can't get type for R type: %d (IO)", type);
-      /* unreached (-Wall): */ memtid = tid;  buf = &tid;
+    errorcall (call, "Can't get type for R type: %d (IO)", type);
+    /* unreached (-Wall): */ memtid = tid;  buf = &tid;
   }
-
+  
   if (H5Sget_simple_extent_dims (space, dims, maxdims) < 0)
     errorcall (call, "Unable to get dimensions of space");
 
@@ -322,7 +343,7 @@ vector_io (SEXP call, int writeflag, hid_t dataset, hid_t space, SEXP obj)
   if (H5Sclose (mspace) < 0)
     errorcall (call, "Unable to close point space");
 
-  if (type == STRSXP) {
+  if (type == STRSXP || type == LGLSXP) {
       if (H5Tclose (memtid) < 0)
 	errorcall (call, "Unable to close string reference type");
     }
@@ -367,11 +388,9 @@ hdf5_save_attributes (SEXP call, hid_t loc_id, SEXP val)
 	}
       else if (type == LGLSXP)
 	{
-	  memtid = H5T_NATIVE_INT;
-	  tid = H5Tcopy (H5T_NATIVE_UINT);
-	  H5Tset_precision (tid, 1);
-	  H5Tset_size (tid, 1);
-	  buf = INTEGER (attr);
+	  memtid = make_boolean_type (call);
+          tid = make_boolean_type (call);
+	  buf = LOGICAL (attr);
 	}
      else if (type == INTSXP)
 	{
@@ -397,15 +416,12 @@ hdf5_save_attributes (SEXP call, hid_t loc_id, SEXP val)
       if (H5Aclose (aid) < 0)
 	errorcall (call, "unable to close attribute `%s'", name);
 
-      if (type == STRSXP)
+      if (type == STRSXP || type == LGLSXP)
 	{
 	  if (H5Tclose (memtid) < 0)
 	    errorcall (call,
 		       "unable to close string reference type `%s'",
 		       name);
-	}
-      if (type == LGLSXP || type == STRSXP)
-	{
 	  if (H5Tclose (tid) < 0)
 	    errorcall (call, "unable to close output type `%s'", name);
 	}
@@ -441,11 +457,7 @@ hdf5_write_vector (SEXP call, hid_t id, const char *symname, SEXP val)
     if (type == STRSXP)
       tid = get_string_type (call, val);
     else if (type == LGLSXP)
-      {
-	tid = H5Tcopy (H5T_NATIVE_UINT);
-	H5Tset_precision (tid, 1);
-	H5Tset_size (tid, 1);
-      }
+      tid = make_boolean_type (call);
     else if (type == INTSXP)
       tid = H5T_NATIVE_INT;
     else if (type == REALSXP)
@@ -569,12 +581,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
       hid_t hdfbooltype;
       size_t size = 0;
 
-      if ((hdfbooltype = H5Tcopy (H5T_NATIVE_UINT)) < 0)
-	errorcall (call, "Cannot copy unsigned integer type");
-      if (H5Tset_precision (hdfbooltype, 1) < 0)
-	errorcall (call, "Cannot set precision of boolean type");
-      if (H5Tset_size (hdfbooltype, 1) < 0)
-	errorcall (call, "Cannot set size of boolean type");
+      hdfbooltype = make_boolean_type (call);
 
       for (pos = 0; pos < colcount; pos++)
 	{
@@ -592,7 +599,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 	      hdftypes[pos] = get_string_type (call, VECTOR_ELT (val, pos));
 	      break;
 	    case LGLSXP:
-	      hdftypes[pos] = hdfbooltype;
+	      hdftypes[pos] = make_boolean_type (call);
 	      break;
 	    default:
 	      errorcall (call,
@@ -672,7 +679,7 @@ hdf5_save_object (SEXP call, hid_t fid, const char *symname, SEXP val)
 		    }
 		    break;
 		  case LGLSXP:
-		    *(unsigned char *)ptr = INTEGER (item)[ri];
+		    *(unsigned char *) ptr = LOGICAL (item)[ri];
 		    break;
 		  default:
 		    abort ();
@@ -1088,8 +1095,16 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
 	  switch (class)
 	    {
 	    case H5T_INTEGER:
-	      type = (tid_size == 1) ? LGLSXP : INTSXP;
-	      memtid = H5Tcopy (H5T_NATIVE_INT);
+	      if (tid_size == 1)
+                {
+                  memtid = make_boolean_type (ainfo->call);
+                  type = LGLSXP;
+                }
+              else
+                {
+                  memtid = H5Tcopy (H5T_NATIVE_INT);
+                  type = INTSXP;
+                }
 	      break;
 	    case H5T_FLOAT:
 	      type = REALSXP;
@@ -1108,7 +1123,10 @@ hdf5_process_attribute (hid_t loc_id, const char *attrName, void *data)
 	  switch (class)
 	    {
 	    case H5T_INTEGER:
-	      buf = INTEGER (vec);
+              if (tid_size == 1)
+                buf = LOGICAL (vec);
+              else
+                buf = INTEGER (vec);
 	      break;
 	    case H5T_FLOAT:
 	      buf = REAL (vec);
@@ -1345,10 +1363,16 @@ hdf5_process_object (hid_t id, const char *name, void *client_data)
                     {
                     case H5T_INTEGER:
                       {
-                        SEXPTYPE type = (csize == 1) ? LGLSXP : INTSXP;
-                        
-                        *rowptr = allocVector (type, rowcount);
-                        VECLOOP (type, INTEGER, H5T_NATIVE_INT);
+                        if (csize == 1)
+                          {
+                            *rowptr = allocVector (LGLSXP, rowcount);
+                            VECLOOP (LGLSXP, LOGICAL, H5T_NATIVE_INT);
+                          }
+                        else 
+                          {
+                            *rowptr = allocVector (INTSXP, rowcount);
+                            VECLOOP (INTSXP, INTEGER, H5T_NATIVE_INT);
+                          }
                       }
                       break;
                     case H5T_FLOAT:
